#
# spec file for package wsl-appx
#
# Copyright (c) 2024 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#
# needssslcertforbuild


# needsappxsslcertforbuild

Name:           wsl-appx
Version:        1
Release:        15.264
Summary:        SUSE on Windows application
License:        MIT
Group:          Productivity/Other
URL:            https://gitlab.suse.de/cbosdonnat/wsl-app
Source0:        AppxManifest.xml
Source1:        openSUSE.tar.gz
Source2:        sle.tar.gz
BuildRequires:  WSL-DistroLauncher
BuildRequires:  WSL-DistroLauncher-debug
BuildRequires:  fb-util-for-appx
BuildRequires:  mingw64-filesystem
%if 0%{?is_opensuse}
BuildRequires:  openSUSE-release
%else
BuildRequires:  sles-release
%endif
BuildRequires:  openssl(cli)
BuildRoot:      %{_tmppath}/%{name}-%{version}-build

%description
Windows Store application providing SLES or openSUSE.

%prep
%setup -Tc -a1 -a2
%if 0%{?is_opensuse}
ln -s openSUSE files
%else
ln -s sle files
%endif

os_release_file="/usr/lib/os-release"
if [ -e "/etc/os-release" ]; then
	os_release_file="/etc/os-release"
fi

# This is dirty. The CN of the AppxManifest needs to match the
# signing certificate's CN. In OBS the CN is the project name. The
# project itself may not have a cert so we can't use %_project. So
# we abuse %vendor which is the project with the key.
#PROJECT="%vendor"
#PROJECT="${PROJECT##*/}"
#OBS="%vendor"
#OBS="${OBS#*//}"
#OBS="${OBS%/*}"
#PUBLISHER="E=$PROJECT@$OBS, CN=$PROJECT OBS Project"
if [ -e "%{_sourcedir}/_projectcert.crt" ]; then
	# mangle value so Windows accepts it
	PUBLISHER=`openssl x509 -subject -in "%{_sourcedir}/_projectcert.crt" -noout -nameopt RFC2253 |sed -e 's/^subject=//;s/emailAddress/E/g;s/,/, /g'`
else
	PUBLISHER="nobody"
fi

# Evaluate os-release file as bash variable assignments
source "$os_release_file"

# PRETTY_NAME on SLE frequently has release label in parentheses as suffix.
# The Microsoft Store rejects this spelling for not matching the display name.
# Trim PRETTY_NAME from space, open paren to end of string:
# SUSE Linux Enterprise Server 15 SP3 (Snapshot 16) -> SUSE Linux Enterprise Server 15 SP3
PRETTY_NAME="${PRETTY_NAME// (*/}"
# Additionally, remove " Server" as we offer a Server|Desktop version at firstboot
PRETTY_NAME="${PRETTY_NAME// Server/}"
APPID="${PRETTY_NAME//[^[:alnum:]]/}"
IDENTITYAPPID="${PRETTY_NAME//[^[:alnum:]\.]/}"
LAUNCHERNAME="${PRETTY_NAME//[^[:alnum:].]/-}.exe"

# These should work unless we move away from:
#   SUSE Linux Enterprise XX SPY
# where:
#   XX = MAJOR_VER
#    Y = SP_VER
#
MAJOR_VER="${PRETTY_NAME:22:2}"
SP_VER="${PRETTY_NAME:27:1}"

# FIX bsc#1179874 Error in parsing the WSL appx package
# PRETTY_NAME in SLES development snapshots can exceed 40 characters allowed in ShortName appx schema field
# Define SHORT_NAME as first 35 characters of PRETTY_NAME and use in AppManifest.xml template
# Length 35 discards development snapshot/beta/rc qualifier cleanly:
# 'SUSE Linux Enterprise Server 15 SP3 (Snapshot11)' -> 'SUSE Linux Enterprise Server 15 SP3'
SHORT_NAME="${PRETTY_NAME::35}"

# RELEASE is used for digits of the appx version submitted to the MS Store.
# To remove the circular dependency on kiwi opensuse-wsl-image or suse-wsl-image,
# set RELEASE here to be an easily matched string of integer digits.
# Substitute these digits with an actual release number during kiwi
# image build using OBS source services.
RELEASE="0.0"
OARCH="%_arch"
ARCH="%_arch"
case "$ARCH" in
	x86_64) ARCH="x64" ;;
	aarch64) ARCH="arm64" ;;
esac
# This has to be "SUSE" right now as per requirements from the
# way the store is set up
PUBLISHER_DISPLAY_NAME="SUSE"
# an appx version needs to match this pattern:
# '(0|[1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])(\.(0|[1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])){3}'
# found in the error log if you don't set the proper version.
# for actual store submission there are more rules:
# https://docs.microsoft.com/en-us/windows/uwp/publish/package-version-numbering
# So we need to mangle the versions in a creative way
# Summarized, the digits of the four segments are limited to the following ranges:
# 0-65535, 0-65535, 0-65535, 0
# Where the fourth segment is reserved for the MS Store use.
if [ "$ID" = "opensuse-tumbleweed" ]; then
	# Concatenate VERSION_ID year and day to consume only one segment 0-65535 e.g. 20210524 -> 21144
	# Retain RELEASE version number in the two remaining segments 0-65535.0-65535
	# Example: VERSION_ID="20210524" date --date=$VERSION_ID +"%y%j" -> 21144
	VERSION=`printf "%d.%d.%d.0" "$(date --date=$VERSION_ID +"%y%j")" "${RELEASE%.*}" "${RELEASE#*.}"`
else
	RELEASE="${RELEASE/lp???./}"
	# Concatenate digits of VERSION_ID to consume only one segment 0-65535 e.g. 15.3 -> 153
	# Retain RELEASE version number in the two remaining segments 0-65535.0-65535
	VERSION=`printf "%d.%d.%d.0" "${VERSION_ID//\./}" "${RELEASE%.*}" "${RELEASE#*.}"`
fi

for i in PRETTY_NAME APPID IDENTITYAPPID OARCH ARCH PUBLISHER PUBLISHER_DISPLAY_NAME VERSION LAUNCHERNAME MAJOR_VER SP_VER SHORT_NAME; do
	eval echo "\"$i='\$$i'\""
done > .settings
cp -v .settings files/DOTsettings

cd files
sed -e "s/@PRETTY_NAME@/${PRETTY_NAME}/g;s/@APPID@/$APPID/g;s/@IDENTITYAPPID@/$IDENTITYAPPID/g;s/@PUBLISHER@/$PUBLISHER/g;s/@PUBLISHER_DISPLAY_NAME@/$PUBLISHER_DISPLAY_NAME/g;s/@VERSION@/${VERSION}/g;s/@LAUNCHERNAME@/$LAUNCHERNAME/g;s/@SHORT_NAME@/$SHORT_NAME/g;s/@ARCH@/$ARCH/g" \
	< %{SOURCE0} > AppxManifest.xml
cat AppxManifest.xml

# Get the binary
cp %{_mingw64_bindir}/DistroLauncher.exe "$LAUNCHERNAME"

# TODO So far .pri files are generated by VS and added to the source
# it would be way better to have them generated here with an opensource tool

%build

%install
mkdir -p $RPM_BUILD_ROOT%{_datadir}/%{name}/{Assets,images}

# Be explicit about what is copied to metadata and where
# Errors about missing contents are useful for debugging
install -D -m 0644 files/Assets/*.png $RPM_BUILD_ROOT%{_datadir}/%{name}/Assets/
install -D -m 0644 files/images/*.ico $RPM_BUILD_ROOT%{_datadir}/%{name}/images/
install -D -m 0644 files/*.pri $RPM_BUILD_ROOT%{_datadir}/%{name}
install -D -m 0644 files/*.xml $RPM_BUILD_ROOT%{_datadir}/%{name}
install -D -m 0755 files/*.exe $RPM_BUILD_ROOT%{_datadir}/%{name}
install -D -m 0644 files/DOTsettings $RPM_BUILD_ROOT%{_datadir}/%{name}

%files
%{_datadir}/%{name}

%changelog
* Thu Jul 11 2024 Scott Bradnick <scott.bradnick@suse.com>
- Adding OARCH variable to 'Include BCI free rpm repository also in SLE WSL'
  in Virtualization:WSL/kiwi-images-wsl
* Tue Aug 22 2023 Scott Bradnick <scott.bradnick@suse.com>
- Adding code to remove " Server" from PRETTY_NAME
  * Should only affect SUSE side.
* Thu Aug 17 2023 Scott Bradnick <scott.bradnick@suse.com>
- To account for /etc/YaST2/products.yaml as part of kiwi-images-wsl,
  use wsl-appx to save off an updated copy of the .settings file it
  was already creating.
  * This file will be sourced by kiwi-images-wsl as it is building
    Distro .appx files in SLE for the MAJOR_VER and SP_VER vars
    used for selecting Server|Desktop versions during firstboot.
* Fri May 28 2021 Jeff Kowalczyk <jkowalczyk@suse.com>
- Set Tumbleweed appx version using same method as SLE and Leap.
  * Concatenate VERSION_ID year and day of year to consume only one
    segment of four segment appx version:
    VERSION_ID="20210524" date --date=$VERSION_ID +"%%y%%j" -> 21144
  * Transforming the TW version date to fit in one segment 0-65535
    is necessary for compliance with the MS Store restriction that
    version numbers must always increase. The two RELEASE digits
    are each given their own segment for this reason.
  * The kiwi generated appx file now contains both TW VERSION_ID
    and the appx version for easier cross reference.
  * As with SLE and Leap set digits two and three of appx VERSION
    to 0 for subsequent regex or template substitution during kiwi
    image build. With the zero digits e.g. 153.0.0.0 it is still
    possible to manually create valid appx images for side load.
  * Remove unused APPXNAME which is now set by kiwi direct appx
    image creation.
* Thu May 13 2021 Jeff Kowalczyk <jkowalczyk@suse.com>
- Remove circular dependency on kiwi WSL image to set release
  number as digits of VERSION passed to AppxManifest.xml template.
  * Previously an aggregate package ensured kiwi
    opensuse-wsl-image or suse-wsl-image were available at
    wsl-appx build time.
  * Now that kiwi directly produces appx images and wsl-appx is
    used only for creating appx metadata, this circular dependency
    should be removed.
  * One option would be to use another package for release number,
    e.g. openSUSE-release.
  * At this time, set digits two and three of appx VERSION to 0
    for subsequent regex or template substitution during kiwi
    image build.
  * A future revision may set digit two and three of appx VERSION
    to this wsl-appx $RELEASE digits so they will increment for
    sideload upgrades, although still are not suitable for
    Microsoft Store upload without kiwi substitution.
  * Kiwi image build must now substitute a release number in
    AppxManifest.xml via OBS source service or other templating
    mechanism available to kiwi.
  * For reference, digits of appx VERSION are limited to the
    following ranges: # 0-65535, 0-65535, 0-65535, 0
* Thu May 13 2021 Jeff Kowalczyk <jkowalczyk@suse.com>
- Fix Tumbleweed appx VERSION assignment. Bash printf %%02d parses
  string dates 08 and 09 as octal so force base 10 with 10#expr.
* Thu May  6 2021 Jeff Kowalczyk <jkowalczyk@suse.com>
- Set appx version for Microsoft Store using first segment for OS
  VERSION_ID, segments two and three for OS image release version.
  * Applies only to SLE and Leap at this time, no change to Tumbleweed
  * printf "%%d.%%d.%%d.0" "${VERSION_ID//\./}" "${RELEASE%%.*}" "${RELEASE#*.}"
  * Preserves image build versions, previous concatenation was lossy
  * Version 153.2.283.0 formerly represented as 15.3.2283.0
- Trim os-release PRETTY_NAME to exclude snapshot or release label
  consistent with Microsoft Store registered display name.
  * PRETTY_NAME on SLE has release label in parentheses
  * Microsoft Store rejects mismatch with display name
  * Trim PRETTY_NAME from space, open paren to end of string
  * SUSE Linux Enterprise Server 15 SP3 (Snapshot 16) becomes
    SUSE Linux Enterprise Server 15 SP3
- When reading os-release variables spell out bash builtin source
  for readability instead of '.'
* Fri Mar  5 2021 Jeff Kowalczyk <jkowalczyk@suse.com>
- wsl-appx package is now metadata input kiwi wsl image type
  * wsl-appx packages metadata for appx utility to consume via kiwi
  * WSL kiwi images require wsl-appx metadata and output appx image type
* Fri Mar  5 2021 Jeff Kowalczyk <jkowalczyk@suse.com>
- Format AppxManifest.xml template for min diff to kiwi output
  * kiwi appx image output emits Unix eol tidy xml with sorted attributes
  * Use tidy --input-xml true --indent yes --indent-spaces 4
  - -sort-attributes alpha --wrap 0 --write-back yes AppxManifest.xml
  * Manually remove space from empty xml elements closing "/>"
    as tidy has no setting for this:
    sed -i -e "s_ />_/>_g" AppxManifest.xml
  * Remove xml declaration standalone="yes" to match kiwi xml output
  * Convert from DOS eol to Unix eol to match kiwi xml output
  * Zero diff between template and kiwi output helps debugging
* Fri Mar  5 2021 Jeff Kowalczyk <jkowalczyk@suse.com>
- Fix bsc#1179874 Error in parsing the WSL appx package
  * Add SHORT_NAME as truncated PRETTY_NAME to comply with AppManifest.xml schema
* Wed Jul  1 2020 Scott Reeves <sreeves@suse.com>
- Change APPID parsing to pass MS store tests. Dont strip
  out the '.' in '15.2' for the Identity->Name
* Wed May  6 2020 Dominique Leuenberger <dimstar@opensuse.org>
- Fix build after the RPM Database change from bdb to ndb: the
  Packages database is now called Packages.db (suse_version > 1500).
* Wed Feb 26 2020 Ludwig Nussel <lnussel@suse.de>
- PUBLISHER_DISPLAY_NAME has to be "SUSE" for the MS store to accept the
  submission
* Fri Feb  7 2020 Ludwig Nussel <lnussel@suse.de>
- set publisher to "SUSE LLC" for SUSE
* Wed Jan 15 2020 Ludwig Nussel <lnussel@suse.de>
- add license to genterated .packages file
* Tue Jan 14 2020 Ludwig Nussel <lnussel@suse.de>
- wsl-launcher was renamed to WSL-DistroLauncher
* Fri Nov 22 2019 Ludwig Nussel <lnussel@suse.de>
- store file with sha256sum of appx file. Otherwise build-compare would
  consider build results unchanged that only affected eg the launcher.
* Wed Nov  6 2019 Ludwig Nussel <lnussel@suse.de>
- use different approach to the version in order to match guidelines
  https://docs.microsoft.com/en-us/windows/uwp/publish/package-version-numbering
* Tue Nov  5 2019 Ludwig Nussel <lnussel@suse.de>
- set publisher display name correctly to SUSE resp openSUSE
- add compat cert requirement for OBS instances that don't
  understand #needsappxsslcertforbuild
* Thu Oct 24 2019 Ludwig Nussel <lnussel@suse.de>
- appx needs a differnt cert, therefore use #needsappxsslcertforbuild
* Thu Oct 17 2019 Ludwig Nussel <lnussel@suse.de>
- use SLE branding on SLE
* Thu Oct 17 2019 Ludwig Nussel <lnussel@suse.de>
- set publisher based on project SSL cert
* Thu Oct 17 2019 Ludwig Nussel <lnussel@suse.de>
- include architecure in appx file name
* Thu Oct 17 2019 Ludwig Nussel <lnussel@suse.de>
- add build number to appx file name
* Thu Oct 10 2019 Ludwig Nussel <lnussel@suse.de>
- rename DistroLauncher.exe to distro specific name
* Tue Oct  8 2019 Ludwig Nussel <lnussel@suse.de>
- try setting a more meaningful version in the appx file
* Fri Oct  4 2019 Ludwig Nussel <lnussel@suse.de>
- actually compress install.tar.gz
- don't needlessly package appx as rpm
* Wed Oct  2 2019 Ludwig Nussel <lnussel@suse.de>
- produce package list and print appx size
* Wed Oct  2 2019 Ludwig Nussel <lnussel@suse.de>
- also check /etc/os-release in case /usr/lib/os-release doesn't exist
* Tue Oct  1 2019 Ludwig Nussel <lnussel@suse.de>
- adjust wsl image name
- prepare for sle
* Wed Sep 11 2019 Ludwig Nussel <lnussel@suse.de>
- add hack to set publisher for signing
* Thu Aug  8 2019 Ludwig Nussel <lnussel@suse.de>
- auto adjust to distro version
* Thu Oct 19 2017 cbosdonnat@suse.com
- Initial OBS build
